//
// b e e p . c
//
// beep with melody on the PC
//
// Jun. 24, 2002 by W.Nishida (http://www.skyfree.org)
//

#include <sys/io.h>		// ioperm(), inb(), outb()
#include <stdio.h>		// fprintf(), stderr
#include <string.h>		// index()
#include <unistd.h>		// usleep()

const int pit_ctl  = 0x43;	// 8253 Programmable Interval Timer (PIT) control
const int pit_cnt2 = 0x42;	// PIT 2nd counter
const int port_b   = 0x61;	// 8255 port B
const int wait     = 0x80;	// 1 microsec wait / access

#define PIT_CMD      0xB6	// PIT control word
				//   Counter 2, LSB->MSB, Square, Binary
				//       10        11       011     0
#define CLK2         1193180	// PIT clock frequency
#define DURATION     200000	// Sound duration (microsec)

// Sound frequency table (Hz)

int snd_tbl[] = {
  264,		// C4
  297,		// D4
  330,		// E4
  352,		// F4
  396,		// G4
  440,		// A4
  495,		// B4
  528,		// C5
  594,		// D5
  660,		// E5
  704,		// F5
  792,		// G5
  880,		// A5
  990,		// B5
  1056,		// C6
  0
 };

const char scale[] = "cdefgabCDEFGAB";

// simple beep function
//	hz: sound frequency
//	duration: sound duration in micorsec

void beep(int hz, int duration) {
  int cnt, last;

  if (hz == 0) {			// Be silent
    usleep(duration);			// Wait
    return;
   }

  // Convert sound frequency to timer2 count

  cnt = CLK2 / hz;

  // Define sound frequency

  outb(PIT_CMD, pit_ctl);		// Send a command to PIT
  inb(wait);				// Wait
  outb(cnt & 0xff, pit_cnt2);		// Counter 2 LSB
  inb(wait);				// Wait
  outb((cnt & 0xff00) >> 8, pit_cnt2);	// Counter 2 MSB

  // Speaker control

  last = inb(port_b);			// Read last output value
  outb(last | 3, port_b);		// Speaker on (GATE2 (PB0) = B (PB1) = H)
  usleep(duration);			// Wait
  outb(last & ~3, port_b);		// Speaker off
 }

int main(int argc, char** argv) {
  int ret, i = 0;
  char *src, *ptr;

  // Allow I/O access against PIT, port B and 0x80

  ret  = ioperm(pit_cnt2, 2, 1);
  ret |= ioperm(port_b, 1, 1);
  ret |= ioperm(wait, 1, 1);
  if (ret) {
    fprintf(stderr, "I/O permission error!\n");
    return 1;
   }

  // Play sound

  if (argc == 1) {			// Play cdefgah...
    while (snd_tbl[ i ])
      beep(snd_tbl[ i++ ], DURATION);
   }
  else {
    src = argv[ 1 ];
    while (*src) {
      ptr = index(scale, (int) *src);
      if (ptr)
        beep(snd_tbl[ ptr - scale ], DURATION);
      else
        beep(0, DURATION);
      src++;
     }
   }
  return 0;
 }
